@@ -48,36 +48,6 @@ class FormatField(Field):
         "Generate fields with formatted values."
         return Field.items(self, *map(self.format, values))
 
-class NumericField(Field):
-    """Field which indexes numbers in a prefix tree.
-    
-    :param name: name of field
-    :param step: precision step
-    """
-    def __init__(self, name, step=None, store=False, index=True):
-        Field.__init__(self, name, store)
-        self.step = step or lucene.NumericUtils.PRECISION_STEP_DEFAULT
-        self.index = index
-    def items(self, *values):
-        "Generate lucene NumericFields suitable for adding to a document."
-        for value in values:
-            field = lucene.NumericField(self.name, self.step, self.store, self.index)
-            if isinstance(value, float):
-                field.doubleValue = value
-            else:
-                field.longValue = long(value)
-            yield field
-    def range(self, start, stop, lower=True, upper=False):
-        "Return lucene NumericRangeQuery."
-        if isinstance(start, float) or isinstance(stop, float):
-            start, stop = (value if value is None else lucene.Double(value) for value in (start, stop))
-            return lucene.NumericRangeQuery.newDoubleRange(self.name, start, stop, lower, upper)
-        if start is not None:
-            start = None if start < lucene.Long.MIN_VALUE else lucene.Long(long(start))
-        if stop is not None:
-            stop = None if start > lucene.Long.MAX_VALUE else lucene.Long(long(stop))
-        return lucene.NumericRangeQuery.newLongRange(self.name, start, stop, lower, upper)
-
 class PrefixField(Field):
     """Field which indexes every prefix of a value into a separate component field.
     The customizable component field names are expressed as slices.
@@ -188,19 +158,29 @@ class DateTimeField(PrefixField):
         queries += [PrefixField.range(self, *item) for item in items[1:-1]]
         queries.append(PrefixField.range(self, items[-1][0], str(stop), upper=upper))
         return Query.any(*queries)
-    def within(self, days=0, weeks=0, utc=False, **delta):
+    def duration(self, date, days=0, **delta):
+        """Return date range query within time span of date.
+        
+        :param date: origin date or tuple
+        :param days, delta: timedelta parameters
+        """
+        if not isinstance(date, datetime.date):
+            date = tuple(date) + (None, 1, 1)[len(date):]
+            date = (datetime.datetime if len(date) > 3 else datetime.date)(*date)
+        delta = datetime.timedelta(days, **delta)
+        return self.range(*sorted([date, date + delta]), upper=True)
+    def within(self, days=0, weeks=0, utc=True, **delta):
         """Return date range query within current time and delta.
         If the delta is an exact number of days, then dates will be used.
         
         :param days, weeks: number of days to offset from today
         :param utc: optionally use utc instead of local time
-        :params delta: additional timedelta parameters
+        :param delta: additional timedelta parameters
         """
-        now = datetime.datetime.utcnow() if utc else datetime.datetime.now()
+        date = datetime.datetime.utcnow() if utc else datetime.datetime.now()
         if not (isinstance(days + weeks, float) or delta):
-            now = now.date()
-        delta = datetime.timedelta(days, weeks=weeks, **delta)
-        return self.range(*sorted([now, now + delta]), upper=True)
+            date = date.date()
+        return self.duration(date, days, weeks=weeks, **delta)
 
 class Document(object):
     """Delegated lucene Document.
