@@ -21,8 +21,77 @@ class Atomic(object):
 Atomic.register(basestring)
 Atomic.register(lucene.TokenStream)
 
+class TokenFilter(lucene.PythonTokenFilter):
+    """Create an iterable lucene TokenFilter from a TokenStream.
+    In lucene version 2, call on any iterable of tokens.
+    In lucene version 3, subclass and override :meth:`incrementToken`;
+    attributes are cached as properties to create a Token interface.
+    """
+    def __init__(self, input):
+        if issubclass(lucene.TokenFilter, collections.Iterable):
+            lucene.PythonTokenFilter.__init__(self, lucene.EmptyTokenStream())
+            self.next = iter(input).next
+        else:
+            lucene.PythonTokenFilter.__init__(self, input)
+            self.input = input
+    def __iter__(self):
+        return self
+    if not hasattr(lucene.TokenFilter, 'next'):
+        def next(self):
+            if self.incrementToken():
+                return self
+            raise StopIteration
+    def incrementToken(self):
+        "Advance to next token and return whether the stream is not empty."
+        return self.input.incrementToken()
+    def __getattr__(self, name):
+        cls = getattr(lucene, name + 'Attribute').class_
+        attr = self.getAttribute(cls) if self.hasAttribute(cls) else self.addAttribute(cls)
+        setattr(self, name, attr)
+        return attr
+    @property
+    def offset(self):
+        "Start and stop character offset."
+        return self.Offset.startOffset(), self.Offset.endOffset()
+    @offset.setter
+    def offset(self, item):
+        self.Offset.setOffset(*item)
+    @property
+    def payload(self):
+        "Payload bytes."
+        data = self.Payload.payload.data
+        return data and data.string_
+    @payload.setter
+    def payload(self, data):
+        self.Payload.payload = lucene.Payload(lucene.JArray_byte(bytes(data)))
+    @property
+    def positionIncrement(self):
+        "Position relative to the previous token."
+        return self.PositionIncrement.positionIncrement
+    @positionIncrement.setter
+    def positionIncrement(self, index):
+        self.PositionIncrement.positionIncrement = index
+    @property
+    def term(self):
+        "Term text."
+        return self.Term.term()
+    @term.setter
+    def term(self, text):
+        self.Term.setTermBuffer(text)
+    @property
+    def type(self):
+        "Lexical type."
+        return self.Type.type()
+    @type.setter
+    def type(self, text):
+        self.Type.setType(text)
+
 class Analyzer(lucene.PythonAnalyzer):
-    "Return a lucene Analyzer which chains together a tokenizer (or analyzer) and filters."
+    """Return a lucene Analyzer which chains together a tokenizer and filters.
+    
+    :param tokenizer: lucene Tokenizer or Analyzer
+    :param filters: lucene TokenFilters or python generators.
+    """
     def __init__(self, tokenizer, *filters):
         lucene.PythonAnalyzer.__init__(self)
         self.tokenizer, self.filters = tokenizer, filters
@@ -30,15 +99,40 @@ class Analyzer(lucene.PythonAnalyzer):
         tokens = self.tokenizer.tokenStream(field, reader) if isinstance(self.tokenizer, lucene.Analyzer) else self.tokenizer(reader)
         for filter in self.filters:
             tokens = filter(tokens)
-        return tokens
+        return tokens if isinstance(tokens, lucene.TokenStream) else TokenFilter(tokens)
+    def tokens(self, text):
+        "Return lucene TokenStream from text."
+        return self.tokenStream('', lucene.StringReader(text))
+    def parse(self, query, field='', op='', version='current', **attrs):
+        """Return parsed lucene Query.
+        
+        :param field: default query field name or sequence of names
+        :param op: default query operator ('or', 'and')
+        :param version: lucene Version string, leave blank for deprecated constructor
+        :param attrs: additional attributes to set on the parser
+        """
+        # parsers aren't thread-safe (nor slow), so create one each time
+        parser = lucene.QueryParser if isinstance(field, basestring) else lucene.MultiFieldQueryParser
+        version = 'LUCENE_' + version.replace('.', '').upper()
+        try:
+            parser = parser(getattr(lucene.Version, version), field, self)
+        except (AttributeError, lucene.InvalidArgsError):
+            parser = parser(field, self) # 2.4 lacks Version, 2.9.0 lacks constructor
+        if op:
+            parser.defaultOperator = getattr(lucene.QueryParser.Operator, op.upper())
+        for name, value in attrs.items():
+            setattr(parser, name, value)
+        if isinstance(parser, lucene.MultiFieldQueryParser):
+            return parser.parse(parser, query) # bug in method binding
+        return parser.parse(query)
 
 class IndexReader(object):
     """Delegated lucene IndexReader, with a mapping interface of ids to document objects.
     
-    :param directory: lucene IndexReader or directory
+    :param reader: lucene IndexReader
     """
-    def __init__(self, directory):
-        self.indexReader = directory if isinstance(directory, lucene.IndexReader) else lucene.IndexReader.open(directory)
+    def __init__(self, reader):
+        self.indexReader = reader
     def __getattr__(self, name):
         if name == 'indexReader':
             raise AttributeError(name)
@@ -51,17 +145,10 @@ class IndexReader(object):
         return itertools.ifilterfalse(self.isDeleted, xrange(self.maxDoc()))
     def __getitem__(self, id):
         return Document(self.document(id))
-    def __delitem__(self, id):
-        self.deleteDocument(id)
     @property
     def directory(self):
         "reader's lucene Directory"
         return self.indexReader.directory()
-    def delete(self, name, value):
-        """Delete documents with given term.
-        Acquires a write lock.  Deleting from an `IndexWriter`_ is encouraged instead.
-        """
-        self.deleteDocuments(lucene.Term(name, value))
     def count(self, name, value):
         "Return number of documents with given term."
         return self.docFreq(lucene.Term(name, value))
@@ -69,9 +156,25 @@ class IndexReader(object):
         "Return field names, given option description."
         option = getattr(self.FieldOption, option.upper())
         return list(self.getFieldNames(option))
-    def terms(self, name, start='', stop=None, counts=False):
-        "Generate a slice of term values, optionally with frequency counts."
-        with contextlib.closing(self.indexReader.terms(lucene.Term(name, start))) as termenum:
+    def terms(self, name, value='', stop=None, counts=False, **fuzzy):
+        """Generate a slice of term values, optionally with frequency counts.
+        Supports a range of terms, wildcard terms, or fuzzy terms.
+        
+        :param name: field name
+        :param value: initial term text or wildcard
+        :param stop: optional upper bound for simple terms
+        :param counts: include frequency counts
+        :param fuzzy: optional keyword arguments for fuzzy terms
+        """
+        term = lucene.Term(name, value)
+        if fuzzy:
+            args = fuzzy.pop('minSimilarity', 0.5), fuzzy.pop('prefixLength', 0)
+            termenum = lucene.FuzzyTermEnum(self.indexReader, term, *args, **fuzzy)
+        elif '*' in value or '?' in value:
+            termenum = lucene.WildcardTermEnum(self.indexReader, term)
+        else:
+            termenum = self.indexReader.terms(term)
+        with contextlib.closing(termenum):
             term = termenum.term()
             while term and term.field() == name:
                 text = term.text()
@@ -92,28 +195,30 @@ class IndexReader(object):
                 doc = termpositions.doc()
                 positions = (termpositions.nextPosition() for n in xrange(termpositions.freq()))
                 if payloads:
-                    array = lucene.JArray_byte('')
-                    yield doc, [(position, ''.join(termpositions.getPayload(array, 0))) \
-                        for position in positions if termpositions.payloadAvailable]
+                    items, array = [], lucene.JArray_byte('')
+                    for position in positions:
+                        if termpositions.payloadAvailable:
+                            data = termpositions.getPayload(array, 0)
+                            items.append((position, data.string_ if hasattr(data, 'string_') else ''.join(data)))
+                    yield doc, items
                 else:
                     yield doc, list(positions)
-    def comparator(self, name, *names, **kwargs):
-        """Return sequence of documents' field values suitable for sorting.
+    def comparator(self, name, type='string', parser=None):
+        """Return cache of field values suitable for sorting.
+        Parsing values into an array is memory optimized.
+        Map values into a list for speed optimization.
         
         :param name: field name
-        :param names: additional names return tuples of values
-        :param default: keyword only default value
+        :param type: type of field values compatible with lucene FieldCache
+        :param parser: callable applied to field values, available in lucene 2.9 or higher
         """
-        if names:
-            return zip(*(self.comparator(name, **kwargs) for name in (name,)+names))
-        values = [kwargs.pop('default', None)] * self.maxDoc()
-        term = lucene.Term(name, '')
-        with contextlib.closing(self.termDocs(**kwargs)) as termdocs:
-            for value in self.terms(name):
-                termdocs.seek(term.createTerm(value))
-                while termdocs.next():
-                    values[termdocs.doc()] = value
-        return values
+        type = getattr(type, '__name__', type).capitalize()
+        method = getattr(lucene.FieldCache.DEFAULT, 'get{0}s'.format(type))
+        if parser is None:
+            return method(self.indexReader, name)
+        bases = getattr(lucene, 'Python{0}Parser'.format(type)),
+        parser = object.__class__('', bases, {'parse' + type: staticmethod(parser)})
+        return method(self.indexReader, name, parser())
     def spans(self, query, positions=False):
         """Generate docs with occurrence counts for a span query.
         
@@ -143,34 +248,37 @@ class IndexReader(object):
                 yield term, map(operator.attrgetter('startOffset', 'endOffset'), tpv.getOffsets(index))
             else:
                 yield term, list(tpv.getTermPositions(index))
+    def morelikethis(self, doc, *fields, **attrs):
+        """Return MoreLikeThis query for document.
+        
+        :param doc: document id or text
+        :param fields: document fields to use, optional for termvectors
+        :param attrs: additional attributes to set on the morelikethis object
+        """
+        mlt = lucene.MoreLikeThis(self.indexReader)
+        mlt.fieldNames = fields or None
+        for name, value in attrs.items():
+            setattr(mlt, name, value)
+        return mlt.like(lucene.StringReader(doc) if isinstance(doc, basestring) else doc)
 
 class Searcher(object):
     "Mixin interface common among searchers."
+    class StandardAnalyzer(lucene.StandardAnalyzer):
+        __del__ = lucene.StandardAnalyzer.close
     def __init__(self, arg, analyzer=None):
         super(Searcher, self).__init__(arg)
-        if not isinstance(analyzer, (lucene.Analyzer, type(None))):
-            warnings.warn("Support for analyzer constructors will be removed.", DeprecationWarning)
-            analyzer = analyzer()
-        self.analyzer = lucene.StandardAnalyzer() if analyzer is None else analyzer
+        if analyzer is None:
+            analyzer = self.StandardAnalyzer(lucene.Version.LUCENE_CURRENT) if hasattr(lucene, 'Version') else self.StandardAnalyzer()
+        self.analyzer = analyzer
     def __del__(self):
         if str(self) != '<null>':
             self.close()
     def __getitem__(self, id):
         return Document(self.doc(id))
-    def parse(self, query, field='', op='', **attrs):
-        """Return lucene parsed Query.
-        
-        :param field: default query field name
-        :param op: default query operator ('or', 'and')
-        :param attrs: additional attributes to set on the parser
-        """
-        # parsers aren't thread-safe (nor slow), so create one each time
-        parser = lucene.QueryParser(field, self.analyzer)
-        if op:
-            parser.defaultOperator = getattr(lucene.QueryParser.Operator, op.upper())
-        for name, value in attrs.items():
-            setattr(parser, name, value)
-        return parser.parse(query)
+    def parse(self, query, *args, **kwargs):
+        if isinstance(query, lucene.Query):
+            return query
+        return Analyzer.parse.im_func(self.analyzer, query, *args, **kwargs)
     def highlight(self, query, text, count=1, span=True, formatter=None, encoder=None, field='', **attrs):
         """Return highlighted text fragments which match the query.
         
@@ -183,8 +291,7 @@ class Searcher(object):
         :param field: default query field name
         :param attrs: additional attributes to set on the highlighter
         """
-        if not isinstance(query, lucene.Query):
-            query = self.parse(query, field)
+        query = self.parse(query, field)
         highlighter = lucene.Highlighter(*filter(None, [formatter, encoder, lucene.QueryScorer(query, field)]))
         for name, value in attrs.items():
             setattr(highlighter, name, value)
@@ -214,12 +321,9 @@ class Searcher(object):
         :param count: maximum number of hits to retrieve
         :param sort: if count is given, lucene Sort parameters, else a callable key
         :param reverse: reverse flag used with sort
-        :param parser: :meth:`parse` options
+        :param parser: :meth:`Analyzer.parse` options
         """
-        if query is None:
-            query = lucene.MatchAllDocsQuery()
-        elif not isinstance(query, lucene.Query):
-            query = self.parse(query, **parser)
+        query = lucene.MatchAllDocsQuery() if query is None else self.parse(query, **parser)
         if not isinstance(filter, (lucene.Filter, type(None))):
             filter = Filter(filter)
         # use custom HitCollector if all results are necessary, otherwise let lucene's TopDocs handle it
@@ -231,8 +335,8 @@ class Searcher(object):
             topdocs = super(Searcher, self).search(query, filter, count)
         else:
             if isinstance(sort, basestring):
-                sort = lucene.Sort(sort, reverse)
-            elif not isinstance(sort, lucene.Sort):
+                sort = lucene.SortField(sort, lucene.SortField.STRING, reverse)
+            if not isinstance(sort, lucene.Sort):
                 sort = lucene.Sort(sort)
             topdocs = super(Searcher, self).search(query, filter, count, sort)
         scoredocs = list(topdocs.scoreDocs)
@@ -242,12 +346,24 @@ class Searcher(object):
 class IndexSearcher(Searcher, lucene.IndexSearcher, IndexReader):
     """Inherited lucene IndexSearcher, with a mixed-in IndexReader.
     
-    :param directory: directory path or lucene Directory
+    :param directory: directory path, lucene Directory, or lucene IndexReader
     :param analyzer: lucene Analyzer, default StandardAnalyzer
     """
     def __init__(self, directory, analyzer=None):
+        self.closing = set()
+        if isinstance(directory, basestring):
+            if hasattr(lucene.FSDirectory, 'open'):
+                directory = lucene.FSDirectory.open(lucene.File(directory))
+            else:
+                directory = lucene.FSDirectory.getDirectory(directory)
+            self.closing.add(directory)
         Searcher.__init__(self, directory, analyzer)
         self.filters = {}
+    def __del__(self):
+        if str(self) != '<null>':
+            self.close()
+        for obj in self.closing:
+            obj.close()
     def facets(self, ids, *keys):
         """Return mapping of document counts for the intersection with each facet.
         
@@ -255,20 +371,20 @@ class IndexSearcher(Searcher, lucene.IndexSearcher, IndexReader):
         :param keys: field names, term tuples, or any keys to previously cached filters
         """
         counts = collections.defaultdict(dict)
-        bits = Filter(ids).bits()
+        ids = Filter(ids)
         for key in keys:
             filters = self.filters.get(key)
             if isinstance(filters, Filter):
-                counts[key] = len(bits & filters.bits(self.indexReader))
+                counts[key] = ids.overlap(filters, self.indexReader)
             elif isinstance(key, basestring):
                 values = self.terms(key) if filters is None else filters
-                counts.update(self.facets(bits, *((key, value) for value in values)))
+                counts.update(self.facets(ids.docIdSet, *((key, value) for value in values)))
             else:
                 name, value = key
                 filters = self.filters.setdefault(name, {})
                 if value not in filters:
                     filters[value] = Query.term(name, value).filter()
-                counts[name][value] = len(bits & filters[value].bits(self.indexReader))
+                counts[name][value] = ids.overlap(filters[value], self.indexReader)
         return dict(counts)
 
 class MultiSearcher(Searcher, lucene.MultiSearcher):
@@ -291,20 +407,28 @@ class IndexWriter(lucene.IndexWriter):
     :param directory: directory path or lucene Directory, default RAMDirectory
     :param mode: file mode (rwa), except updating (+) is implied
     :param analyzer: lucene Analyzer, default StandardAnalyzer
-    :param mfl: MaxFieldLength, default IndexWriter.DEFAULT_MAX_FIELD_LENGTH
     """
     __len__ = lucene.IndexWriter.numDocs
-    __del__ = Searcher.__del__.im_func
+    __del__ = IndexSearcher.__del__.im_func
     parse = Searcher.parse.im_func
-    def __init__(self, directory=None, mode='a', analyzer=None, mfl=10000):
-        if not isinstance(analyzer, (lucene.Analyzer, type(None))):
-            warnings.warn("Support for analyzer constructors will be removed.", DeprecationWarning)
-            analyzer = analyzer()
-        args = [(directory or lucene.RAMDirectory()), (lucene.StandardAnalyzer() if analyzer is None else analyzer)]
-        if mode != 'a':
-            args.append(mode == 'w')
-        args.append(mfl if isinstance(mfl, self.MaxFieldLength) else self.MaxFieldLength(mfl))
-        lucene.IndexWriter.__init__(self, *args)
+    def __init__(self, directory=None, mode='a', analyzer=None):
+        self.closing = set()
+        if analyzer is None:
+            analyzer = lucene.StandardAnalyzer(lucene.Version.LUCENE_CURRENT) if hasattr(lucene, 'Version') else lucene.StandardAnalyzer()
+            self.closing.add(analyzer)
+        if not isinstance(directory, lucene.Directory):
+            if directory is None:
+                directory = lucene.RAMDirectory()
+            elif hasattr(lucene.FSDirectory, 'open'):
+                directory = lucene.FSDirectory.open(lucene.File(directory))
+            else:
+                directory = lucene.FSDirectory.getDirectory(directory)
+            self.closing.add(directory)
+        mfl = lucene.IndexWriter.MaxFieldLength.LIMITED
+        if mode == 'a':
+            lucene.IndexWriter.__init__(self, directory, analyzer, mfl)
+        else:
+            lucene.IndexWriter.__init__(self, directory, analyzer, (mode == 'w'), mfl)
         self.fields = {}
     @property
     def segments(self):
@@ -338,22 +462,25 @@ class IndexWriter(lucene.IndexWriter):
         """Remove documents which match given query or term.
         
         :param query: :meth:`Searcher.search` compatible query, or optimally a name and value
-        :param options: additional :meth:`parse` options
+        :param options: additional :meth:`Analyzer.parse` options
         """
         if len(query) == 1:
-            query, = query
-            if not isinstance(query, lucene.Query):
-                query = self.parse(query, **options)
-            self.deleteDocuments(query)
+            self.deleteDocuments(self.parse(*query, **options))
         else:
             self.deleteDocuments(lucene.Term(*query))
     def __iadd__(self, directory):
         "Add directory (or reader, searcher, writer) to index."
         if isinstance(directory, basestring):
-            directory = lucene.FSDirectory.getDirectory(directory)
-        elif not isinstance(directory, lucene.Directory):
-            directory = directory.directory
-        self.addIndexesNoOptimize([directory])
+            if hasattr(lucene.FSDirectory, 'open'):
+                directory = lucene.FSDirectory.open(lucene.File(directory))
+            else:
+                directory = lucene.FSDirectory.getDirectory(directory)
+            with contextlib.closing(directory):
+                self.addIndexesNoOptimize([directory])
+        else:
+            if not isinstance(directory, lucene.Directory):
+                directory = directory.directory
+            self.addIndexesNoOptimize([directory])
         return self
 
 class Indexer(IndexWriter):
@@ -377,4 +504,5 @@ class Indexer(IndexWriter):
         "Commit writes and refresh searcher.  Not thread-safe."
         IndexWriter.commit(self)
         if not self.current:
-            self.indexSearcher = IndexSearcher(self.directory, self.analyzer)
+            searcher = self.indexSearcher = IndexSearcher(self.reopen(), self.analyzer)
+            searcher.closing.add(searcher.indexReader)
