@@ -2,18 +2,36 @@ import unittest
 import os, optparse
 import tempfile, shutil
 import itertools
+import collections
 import warnings
-from datetime import date
+import datetime
+import math
 import lucene
 from lupyne import engine
 import fixture
 
+numeric = hasattr(lucene, 'NumericField')
+parser = optparse.OptionParser()
+parser.add_option("-v", "--verbose", action="store_true", dest="verbose")
+options, args = parser.parse_args()
+
+if issubclass(lucene.TokenFilter, collections.Iterable):
+    def typeAsPayload(tokens):
+        "Generator variant of lucene TypeAsPayloadTokenFilter."
+        for token in tokens:
+            token.payload = lucene.Payload(lucene.JArray_byte(bytes(token.type())))
+            yield token
+else:
+    class typeAsPayload(engine.TokenFilter):
+        "Custom implementation of lucene TypeAsPayloadTokenFilter."
+        def incrementToken(self):
+            result = engine.TokenFilter.incrementToken(self)
+            self.payload = self.type
+            return result
+
 class BaseTest(unittest.TestCase):
     def setUp(self):
         self.tempdir = tempfile.mkdtemp(dir=os.path.dirname(__file__))
-        parser = optparse.OptionParser()
-        parser.add_option("-v", "--verbose", action="store_true", dest="verbose")
-        options, args = parser.parse_args()
         self.verbose = options.verbose
     def tearDown(self):
         shutil.rmtree(self.tempdir)
@@ -22,23 +40,25 @@ class TestCase(BaseTest):
     
     def testInterface(self):
         "Indexer and document interfaces."
-        assert engine.IndexWriter.__init__.im_func.func_defaults[-1] == lucene.IndexWriter.DEFAULT_MAX_FIELD_LENGTH
-        with warnings.catch_warnings(record=True) as deprecations:
-            writer = engine.IndexWriter(analyzer=lucene.StandardAnalyzer)
-            engine.IndexSearcher(writer.directory, analyzer=lucene.WhitespaceAnalyzer)
-        assert len(deprecations) == 2
-        Stemmer = engine.Analyzer(lucene.StandardAnalyzer(), lucene.PorterStemFilter, lucene.TypeAsPayloadTokenFilter)
-        indexer = engine.Indexer(analyzer=Stemmer)
+        analyzer = lucene.StandardAnalyzer(lucene.Version.LUCENE_CURRENT) if hasattr(lucene, 'Version') else lucene.StandardAnalyzer()
+        stemmer = engine.Analyzer(analyzer, lucene.PorterStemFilter, typeAsPayload)
+        assert [token.termText() if isinstance(token, lucene.Token) else token.term for token in stemmer.tokens('hello worlds')] == ['hello', 'world']
+        assert str(stemmer.parse('hellos', field=['body', 'title'])) == 'body:hello title:hello'
+        indexer = engine.Indexer(analyzer=stemmer)
         self.assertRaises(lucene.JavaError, engine.Indexer, indexer.directory)
         indexer.set('text')
         indexer.set('name', store=True, index=False, boost=2.0)
         for field in indexer.fields['name'].items('sample'):
             assert isinstance(field, lucene.Field) and field.boost == 2.0
         indexer.set('tag', store=True, index=True)
+        searcher = indexer.indexSearcher
+        indexer.commit()
+        assert searcher is indexer.indexSearcher
         indexer.add(text='hello worlds', name='sample', tag=['python', 'search'])
         assert len(indexer) == 1 and list(indexer) == []
         assert not indexer.optimized
         indexer.commit()
+        assert searcher is not indexer.indexSearcher
         searcher = engine.ParallelMultiSearcher([indexer.indexSearcher, indexer.directory])
         assert searcher.count() == 2 * len(indexer)
         assert list(indexer) == [0]
@@ -93,8 +113,6 @@ class TestCase(BaseTest):
         indexer += temp.directory
         indexer += self.tempdir
         assert len(indexer) == 3
-        x, y = engine.Filter([0, 1]).bits(), engine.Filter([1, 2]).bits()
-        assert list(x & y) == [1] and list(x | y) == [0, 1, 2] and list(x -y) == [0]
         indexer.add(text=lucene.WhitespaceTokenizer(lucene.StringReader('?')))
         indexer.commit()
         assert list(indexer.terms('text')) == ['?']
@@ -115,6 +133,8 @@ class TestCase(BaseTest):
         assert sorted(map(int, articles)) == range(1, 8)
         assert sorted(map(int, indexer.terms('amendment'))) == range(1, 28)
         assert list(itertools.islice(indexer.terms('text', 'right'), 2)) == ['right', 'rights']
+        assert list(indexer.terms('text', 'right*')) == ['right', 'rights']
+        assert list(indexer.terms('text', 'right', minSimilarity=0.5)) == ['eight', 'right', 'rights']
         word, count = next(indexer.terms('text', 'people', counts=True))
         assert word == 'people' and count == 8
         docs = dict(indexer.docs('text', 'people', counts=True))
@@ -127,20 +147,25 @@ class TestCase(BaseTest):
         assert sorted(hit.dict()) == ['__id__', '__score__', 'article']
         hits = indexer.search('people', field='text')
         assert len(hits) == hits.count == 8
+        assert set(map(type, hits.ids)) == set([int]) and set(map(type, hits.scores)) == set([float])
         ids = hits.ids
         hits = indexer.search('people', count=5, field='text')
         assert hits.ids == ids[:len(hits)]
         assert len(hits) == 5 and hits.count == 8
-        hits = indexer.search('text:people', count=5, sort='amendment')
-        assert sorted(hits.ids) == hits.ids and hits.ids != ids[:len(hits)]
-        comparator = map(int, indexer.comparator('amendment', default='0'))
+        assert not any(map(math.isnan, hits.scores))
+        hits = indexer.search('text:people', count=5, sort=lucene.SortField('amendment', lucene.SortField.INT))
+        assert [hit.get('amendment') for hit in hits] == [None, None, '1', u'2', u'4']
+        if lucene.VERSION >= '2.9':
+            assert all(map(math.isnan, hits.scores))
+        parser = (lambda value: int(value or -1)) if hasattr(lucene, 'PythonIntParser') else None
+        comparator = indexer.comparator('amendment', type=int, parser=parser)
         hits = indexer.search('text:people', sort=comparator.__getitem__)
         assert sorted(hits.ids) == hits.ids and hits.ids != ids
-        comparator = indexer.comparator('article', 'amendment')
+        comparator = zip(*map(indexer.comparator, ['article', 'amendment']))
         hits = indexer.search('text:people', sort=comparator.__getitem__)
         assert sorted(hits.ids) != hits.ids
         hits = indexer.search('text:people', count=5, sort='amendment', reverse=True)
-        assert sorted(hits.ids, reverse=True) == hits.ids and hits.ids != ids[:len(hits)]
+        assert [hit['amendment'] for hit in hits] == ['9', '4', '2', '17', '10']
         hit, = indexer.search('freedom', field='text')
         assert hit['amendment'] == '1'
         assert sorted(hit.dict()) == ['__id__', '__score__', 'amendment', 'date']
@@ -185,18 +210,27 @@ class TestCase(BaseTest):
         assert indexer.count(queries[0] - near) == count
         near = queries[0].near(queries[1] | queries[2], slop=1)
         assert indexer.count(queries[0] - near) == count - 1
-        assert indexer.count(queries[0][:100]) == count - 1
+        assert 0 < indexer.count(queries[0][:100]) < count
         spans = dict(indexer.spans(queries[0]))
         assert len(spans) == count and spans == dict(indexer.docs('text', 'persons', counts=True))
         near = queries[0].near(queries[1], slop=2)
         (id, positions), = indexer.spans(near, positions=True)
-        assert indexer[id]['amendment'] == '4' and positions == [(3, 6)]
+        assert indexer[id]['amendment'] == '4' and positions in ([(3, 6)], [(10, 13)])
         with warnings.catch_warnings(record=True) as leaks:
             assert 'persons' in indexer.termvector(id, 'text')
             assert dict(indexer.termvector(id, 'text', counts=True))['persons'] == 2
-            assert dict(indexer.positionvector(id, 'text'))['persons'] == [3, 26]
+            assert dict(indexer.positionvector(id, 'text'))['persons'] in ([3, 26], [10, 48])
             assert dict(indexer.positionvector(id, 'text', offsets=True))['persons'] == [(46, 53), (301, 308)]
         assert len(leaks) == 2 * (lucene.VERSION <= '2.4.1')
+        query = indexer.morelikethis(0)
+        assert str(query) == 'text:united text:states'
+        hits = indexer.search(query & engine.Query.prefix('article', ''))
+        assert len(hits) == 8 and hits[0]['article'] == 'Preamble'
+        assert str(indexer.morelikethis(0, 'article')) == ''
+        assert str(indexer.morelikethis(0, minDocFreq=3)) == 'text:establish text:united text:states'
+        assert str(indexer.morelikethis('jury', 'text', minDocFreq=4, minTermFreq=1)) == 'text:jury'
+        assert str(indexer.morelikethis('jury', 'article')) == ''
+        self.assertRaises(lucene.JavaError, indexer.morelikethis, 'jury')
         del indexer
         assert engine.Indexer(self.tempdir)
     
@@ -254,50 +288,50 @@ class TestCase(BaseTest):
     
     def testSpatial(self):
         "Spatial tile test."
-        indexer = engine.Indexer(self.tempdir)
-        for name, params in fixture.zipcodes.fields.items():
-            indexer.set(name, **params)
-        field = indexer.fields['tile'] = engine.PointField('tile', precision=15, start=10, step=2, store=True)
-        points = []
-        for doc in fixture.zipcodes.docs():
-            if doc['state'] == 'CA':
-                lat, lng = doc.pop('latitude'), doc.pop('longitude')
-                indexer.add(doc, tile=[(lng, lat)], latitude=str(lat), longitude=str(lng))
-                if doc['city'] == 'Los Angeles':
-                    points.append((lng, lat))
-        assert len(list(engine.PolygonField('', precision=15).items(points))) > len(points)
-        indexer.commit()
-        city, zipcode, tile = 'Beverly Hills', '90210', '023012311120332'
-        hit, = indexer.search('zipcode:' + zipcode)
-        assert hit['tile'] == tile and hit['city'] == city
-        hit, = indexer.search(field.prefix(tile))
-        assert hit['zipcode'] == zipcode and hit['city'] == city
-        x, y = (float(hit[l]) for l in ['longitude', 'latitude'])
-        assert field.coords(tile[:4]) == (2, 9)
-        bottom, left, top, right = field.decode(tile)
-        assert left < x < right and bottom < y < top
-        hits = indexer.search(field.near(x, y))
-        cities = set(hit['city'] for hit in hits)
-        assert set([city]) == cities
-        hits = indexer.search(field.near(x, y, precision=10))
-        cities = set(hit['city'] for hit in hits)
-        assert city in cities and len(cities) > 10
-        hits = indexer.search(field.within(x, y, 10**4))
-        cities = set(hit['city'] for hit in hits)
-        assert city in cities and 100 > len(cities) > 50
-        hits = indexer.search(field.within(x, y, 10**5))
-        cities = set(hit['city'] for hit in hits)
-        assert city in cities and len(cities) > 100
+        for PointField in [engine.PointField] + [engine.numeric.PointField] * numeric:
+            indexer = engine.Indexer(self.tempdir, 'w')
+            for name, params in fixture.zipcodes.fields.items():
+                indexer.set(name, **params)
+            field = indexer.fields['tile'] = PointField('tile', precision=15, step=2, store=True)
+            points = []
+            for doc in fixture.zipcodes.docs():
+                if doc['state'] == 'CA':
+                    lat, lng = doc.pop('latitude'), doc.pop('longitude')
+                    indexer.add(doc, tile=[(lng, lat)], latitude=str(lat), longitude=str(lng))
+                    if doc['city'] == 'Los Angeles':
+                        points.append((lng, lat))
+            assert len(list(engine.PolygonField('', precision=15).items(points))) > len(points)
+            indexer.commit()
+            city, zipcode, tile = 'Beverly Hills', '90210', '023012311120332'
+            hit, = indexer.search('zipcode:' + zipcode)
+            assert (hit['tile'] == tile or int(hit['tile']) == int(tile, 4)) and hit['city'] == city
+            hit, = indexer.search(field.prefix(tile))
+            assert hit['zipcode'] == zipcode and hit['city'] == city
+            x, y = (float(hit[l]) for l in ['longitude', 'latitude'])
+            assert field.coords(tile[:4]) == (2, 9)
+            bottom, left, top, right = field.decode(tile)
+            assert left < x < right and bottom < y < top
+            hits = indexer.search(field.near(x, y))
+            cities = set(hit['city'] for hit in hits)
+            assert set([city]) == cities
+            hits = indexer.search(field.near(x, y, precision=10))
+            cities = set(hit['city'] for hit in hits)
+            assert city in cities and len(cities) > 10
+            query = field.within(x, y, 10**4)
+            assert issubclass(PointField, engine.PointField) or len(query) < 3
+            cities = set(hit['city'] for hit in indexer.search(query))
+            assert city in cities and 100 > len(cities) > 50
+            hits = indexer.search(field.within(x, y, 10**5))
+            cities = set(hit['city'] for hit in hits)
+            assert city in cities and len(cities) > 100
+            del indexer
     
     def testFields(self):
         "Custom field tests."
         indexer = engine.Indexer(self.tempdir)
         indexer.set('amendment', engine.FormatField, format='{0:02n}', store=True)
         indexer.set('date', engine.DateTimeField, store=True)
-        try:
-            indexer.set('size', engine.NumericField, store=True)
-        except AttributeError:
-            indexer.set('size', engine.FormatField, format='{0:04n}', store=True)
+        indexer.set('size', engine.FormatField, format='{0:04n}', store=True)
         for doc in fixture.constitution.docs():
             if 'amendment' in doc:
                 indexer.add(amendment=int(doc['amendment']), date=doc['date'], size=len(doc['text']))
@@ -306,40 +340,82 @@ class TestCase(BaseTest):
         assert indexer.count(query) == 9
         query = engine.Query.prefix('amendment', '0')
         assert indexer.count(query) == 9
-        query = indexer.fields['date'].range('', '1921-12', lower=False, upper=True)
+        field = indexer.fields['date']
+        query = field.prefix('1791-12-15')
+        assert indexer.count(query) == 10
+        query = field.range('', '1921-12', lower=False, upper=True)
         assert str(query) == 'date:Y:{ TO 1921} date:Ym:[1921 TO 1921-12]'
         assert indexer.count(query) == 19
-        query = indexer.fields['date'].range(date(1919, 1, 1), date(1921, 12, 31))
+        query = field.range(datetime.date(1919, 1, 1), datetime.date(1921, 12, 31))
         cls = lucene.TermRangeQuery if hasattr(lucene, 'TermRangeQuery') else lucene.ConstantScoreRangeQuery
         fields = [cls.cast_(clause.query).field for clause in query]
         assert fields == ['date:Ymd', 'date:Ym', 'date:Y', 'date:Ym', 'date:Ymd']
         hits = indexer.search(query)
         assert [hit['amendment'] for hit in hits] == ['18', '19']
         assert [hit['date'].split('-')[0] for hit in hits] == ['1919', '1920']
-        query = indexer.fields['date'].within(seconds=100)
+        query = field.within(seconds=100)
         assert indexer.count(query) == 0
-        query = indexer.fields['date'].within(-100*365)
+        query = field.duration([2009], days=-100*365)
         assert 0 < len(query) <= 5
-        assert 0 < indexer.count(query) <= 12
-        assert len(indexer.fields['date'].within(-100)) <= 3
-        assert len(indexer.fields['date'].within(-100.0)) > 3
-        assert len(indexer.fields['date'].within(-100, seconds=1, utc=True)) > 3
-        field, numeric = indexer.fields['size'], hasattr(engine, 'NumericField')
-        assert numeric == (len(list(indexer.terms('size'))) > len(indexer))
+        assert indexer.count(query) == 12
+        assert len(field.within(-100)) <= 3
+        assert len(field.within(-100.0)) > 3
+        assert len(field.within(-100, seconds=1, utc=True)) > 3
+        field = indexer.fields['size']
         sizes = dict((id, int(indexer[id]['size'])) for id in indexer)
         ids = sorted((id for id in sizes if sizes[id] >= 1000), key=sizes.get)
-        query = field.range(1000, None) if numeric else engine.Query.range('size', '1000', None)
+        query = engine.Query.range('size', '1000', None)
         hits = indexer.search(query, sort=sizes.get)
         assert hits.ids == ids
         hits = indexer.search(query, count=3, sort=lucene.SortField('size', lucene.SortField.LONG))
         assert hits.ids == ids[:len(hits)]
-        query = field.range(None, 1000) if numeric else engine.Query.range('size', None, '1000')
+        query = engine.Query.range('size', None, '1000')
         assert indexer.count(query) == len(sizes) - len(ids)
-        if numeric:
+    
+    if numeric:
+        def testNumericFields(self):
+            "Numeric variant fields."
+            indexer = engine.Indexer(self.tempdir)
+            indexer.set('amendment', engine.numeric.NumericField, store=True)
+            indexer.set('date', engine.numeric.DateTimeField, store=True)
+            indexer.set('size', engine.numeric.NumericField, store=True)
+            for doc in fixture.constitution.docs():
+                if 'amendment' in doc:
+                    indexer.add(amendment=int(doc['amendment']), date=[map(int, doc['date'].split('-'))], size=len(doc['text']))
+            indexer.commit()
+            query = indexer.fields['amendment'].range(None, 10)
+            assert indexer.count(query) == 9
+            field = indexer.fields['date']
+            query = field.prefix((1791, 12))
+            assert indexer.count(query) == 10
+            query = field.prefix(datetime.date(1791, 12, 15))
+            assert indexer.count(query) == 10
+            query = field.range(None, (1921, 12), lower=False, upper=True)
+            assert indexer.count(query) == 19
+            query = field.range(datetime.date(1919, 1, 1), datetime.date(1921, 12, 31))
+            hits = indexer.search(query)
+            assert [hit['amendment'] for hit in hits] == ['18', '19']
+            assert [datetime.datetime.utcfromtimestamp(float(hit['date'])).year for hit in hits] == [1919, 1920]
+            query = field.within(seconds=100)
+            assert indexer.count(query) == 0
+            query = field.duration([2009], days=-100*365)
+            assert indexer.count(query) == 12
+            field = indexer.fields['size']
+            assert len(list(indexer.terms('size'))) > len(indexer)
+            sizes = dict((id, int(indexer[id]['size'])) for id in indexer)
+            ids = sorted((id for id in sizes if sizes[id] >= 1000), key=sizes.get)
+            query = field.range(1000, None)
+            hits = indexer.search(query, sort=sizes.get)
+            assert hits.ids == ids
+            hits = indexer.search(query, count=3, sort=lucene.SortField('size', lucene.SortField.LONG))
+            assert hits.ids == ids[:len(hits)]
+            query = field.range(None, 1000)
+            assert indexer.count(query) == len(sizes) - len(ids)
             self.assertRaises(OverflowError, list, field.items(-2**64))
             nf, = field.items(0.5)
             assert nf.numericValue.doubleValue() == 0.5
             assert str(field.range(-2**64, 0)) == 'size:[* TO 0}'
+            assert str(field.range(0, 2**64)) == 'size:[0 TO *}'
             assert str(field.range(0.5, None, upper=True)) == 'size:[0.5 TO *]'
     
     def testHighlighting(self):
